(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", 'dojo-shim/Map', 'dojo-shim/WeakMap'], factory);
    }
})(function (require, exports) {
    "use strict";
    var Map_1 = require('dojo-shim/Map');
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var noop = function () { };
    /**
     * A registry of values, mapped by identities.
     */
    var IdentityRegistry = (function () {
        function IdentityRegistry() {
            this._entryMap = new Map_1.default();
            this._idMap = new WeakMap_1.default();
        }
        /**
         * Look up a value by its identifier.
         *
         * Throws if no value has been registered for the given identifier.
         *
         * @param id The identifier
         * @return The value
         */
        IdentityRegistry.prototype.get = function (id) {
            if (!this.hasId(id)) {
                throw new Error("Could not find a value for identity '" + id.toString() + "'");
            }
            return this._entryMap.get(id).value;
        };
        /**
         * Determine whether the value has been registered.
         * @param value The value
         * @return `true` if the value has been registered, `false` otherwise
         */
        IdentityRegistry.prototype.contains = function (value) {
            return this._idMap.has(value);
        };
        /**
         * Remove from the registry the value for a given identifier.
         * @param id The identifier
         * @return `true` if the value was removed, `false` otherwise
         */
        IdentityRegistry.prototype.delete = function (id) {
            if (!this._entryMap.has(id)) {
                return false;
            }
            var handle = this._entryMap.get(id).handle;
            handle.destroy();
            return true;
        };
        /**
         * Determine whether a value has been registered for the given identifier.
         * @param id The identifier
         * @return `true` if a value has been registered, `false` otherwise
         */
        IdentityRegistry.prototype.hasId = function (id) {
            return this._entryMap.has(id);
        };
        /**
         * Look up the identifier for which the given value has been registered.
         *
         * Throws if the value hasn't been registered.
         *
         * @param value The value
         * @return The identifier otherwise
         */
        IdentityRegistry.prototype.identify = function (value) {
            if (!this.contains(value)) {
                throw new Error('Could not identify non-registered value');
            }
            return this._idMap.get(value);
        };
        /**
         * Register a new value with a new identity.
         *
         * Throws if a different value has already been registered for the given identity,
         * or if the value has already been registered with a different identity.
         *
         * @param id The identifier
         * @param value The value
         * @return A handle for deregistering the value. Note that when called repeatedly with
         *   the same identifier and value combination, the same handle is returned
         */
        IdentityRegistry.prototype.register = function (id, value) {
            var _this = this;
            var existingValue = this.hasId(id) ? this.get(id) : null;
            if (existingValue && existingValue !== value) {
                var str = id.toString();
                throw new Error("A value has already been registered for the given identity (" + str + ")");
            }
            var existingId = this.contains(value) ? this.identify(value) : null;
            if (existingId && existingId !== id) {
                var str = existingId.toString();
                throw new Error("The value has already been registered with a different identity (" + str + ")");
            }
            // Adding the same value with the same id is a noop, return the original handle.
            if (existingValue && existingId) {
                return this._entryMap.get(id).handle;
            }
            var handle = {
                destroy: function () {
                    handle.destroy = noop;
                    _this._entryMap.delete(id);
                    _this._idMap.delete(value);
                }
            };
            this._entryMap.set(id, { handle: handle, value: value });
            this._idMap.set(value, id);
            return handle;
        };
        return IdentityRegistry;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = IdentityRegistry;
    ;
});
//# sourceMappingURL=IdentityRegistry.js.map