(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", 'dojo-compose/compose', 'dojo-core/lang', 'dojo-shim/Promise', 'dojo-shim/Set', 'dojo-shim/Symbol', 'dojo-shim/WeakMap', './IdentityRegistry', './lib/factories', './lib/InstanceRegistry', './lib/makeIdGenerator', './lib/moduleResolver', './lib/realizeCustomElements', './lib/RegistryProvider'], factory);
    }
})(function (require, exports) {
    "use strict";
    var compose_1 = require('dojo-compose/compose');
    var lang_1 = require('dojo-core/lang');
    var Promise_1 = require('dojo-shim/Promise');
    var Set_1 = require('dojo-shim/Set');
    var Symbol_1 = require('dojo-shim/Symbol');
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var IdentityRegistry_1 = require('./IdentityRegistry');
    var factories_1 = require('./lib/factories');
    var InstanceRegistry_1 = require('./lib/InstanceRegistry');
    var makeIdGenerator_1 = require('./lib/makeIdGenerator');
    var moduleResolver_1 = require('./lib/moduleResolver');
    var realizeCustomElements_1 = require('./lib/realizeCustomElements');
    var RegistryProvider_1 = require('./lib/RegistryProvider');
    exports.RegistryProvider = RegistryProvider_1.default;
    /**
     * Identifier for the default action store, if any.
     */
    exports.DEFAULT_ACTION_STORE = Symbol_1.default('Identifier for default action stores');
    /**
     * Identifier for the default widget store, if any.
     */
    exports.DEFAULT_WIDGET_STORE = Symbol_1.default('Identifier for default widget stores');
    var generateWidgetId = makeIdGenerator_1.default('app-widget-');
    var noop = function () { };
    var actionFactories = new WeakMap_1.default();
    var customElementFactories = new WeakMap_1.default();
    var identifiers = new WeakMap_1.default();
    var storeFactories = new WeakMap_1.default();
    var widgetFactories = new WeakMap_1.default();
    var instanceRegistries = new WeakMap_1.default();
    var midResolvers = new WeakMap_1.default();
    var publicRegistries = new WeakMap_1.default();
    var registryProviders = new WeakMap_1.default();
    var widgetInstances = new WeakMap_1.default();
    function addIdentifier(app, id) {
        var set = identifiers.get(app);
        if (set.has(id)) {
            throw new Error("'" + id + "' has already been used as an identifier");
        }
        set.add(id);
        return {
            destroy: function () {
                this.destroy = noop;
                set.delete(id);
            }
        };
    }
    function createCustomWidget(app, id) {
        var customFactories = customElementFactories.get(app);
        var registryProvider = app.registryProvider, stateFrom = app.defaultActionStore;
        return app.defaultWidgetStore.get(id).then(function (state) {
            var options = { id: id, stateFrom: stateFrom, registryProvider: registryProvider, state: state };
            var customFactory = customFactories.get(state.type);
            return customFactory(options);
        }).then(function (widget) {
            widget.own(registerInstance(app, widget, id));
            return widget;
        });
    }
    function registerInstance(app, instance, id) {
        // Maps the instance to its ID
        var instanceHandle = instanceRegistries.get(app).addWidget(instance, id);
        // Maps the ID to the instance
        var idHandle = widgetInstances.get(app).register(id, instance);
        return {
            destroy: function () {
                this.destroy = noop;
                instanceHandle.destroy();
                idHandle.destroy();
            }
        };
    }
    var createApp = compose_1.default({
        set defaultActionStore(store) {
            instanceRegistries.get(this).addStore(store, exports.DEFAULT_ACTION_STORE);
            storeFactories.get(this).register(exports.DEFAULT_ACTION_STORE, function () { return store; });
        },
        get defaultActionStore() {
            var factories = storeFactories.get(this);
            if (factories.hasId(exports.DEFAULT_ACTION_STORE)) {
                return factories.get(exports.DEFAULT_ACTION_STORE)();
            }
            else {
                return null;
            }
        },
        set defaultWidgetStore(store) {
            instanceRegistries.get(this).addStore(store, exports.DEFAULT_WIDGET_STORE);
            storeFactories.get(this).register(exports.DEFAULT_WIDGET_STORE, function () { return store; });
        },
        get defaultWidgetStore() {
            var factories = storeFactories.get(this);
            if (factories.hasId(exports.DEFAULT_WIDGET_STORE)) {
                return factories.get(exports.DEFAULT_WIDGET_STORE)();
            }
            else {
                return null;
            }
        },
        get registryProvider() {
            return registryProviders.get(this);
        },
        registerAction: function (id, action) {
            var _this = this;
            var idHandle = addIdentifier(this, id);
            var instanceHandle = instanceRegistries.get(this).addAction(action, id);
            var promise = new Promise_1.default(function (resolve) {
                resolve(action.configure(publicRegistries.get(_this)));
            }).then(function () { return action; });
            var registryHandle = actionFactories.get(this).register(id, function () { return promise; });
            return {
                destroy: function () {
                    this.destroy = noop;
                    idHandle.destroy();
                    instanceHandle.destroy();
                    registryHandle.destroy();
                }
            };
        },
        registerActionFactory: function (id, factory) {
            var _this = this;
            var idHandle = addIdentifier(this, id);
            var destroyed = false;
            var instanceHandle;
            var registryHandle = actionFactories.get(this).register(id, function () {
                var promise = Promise_1.default.resolve()
                    .then(function () {
                    // Always call the factory in a future turn. This harmonizes behavior regardless of whether the
                    // factory is registered through this method or loaded from a definition.
                    return factory(publicRegistries.get(_this), _this.defaultActionStore);
                })
                    .then(function (action) {
                    if (!destroyed) {
                        instanceHandle = instanceRegistries.get(_this).addAction(action, id);
                    }
                    // Configure the action, allow for a promise to be returned.
                    return Promise_1.default.resolve(action.configure(publicRegistries.get(_this))).then(function () {
                        return action;
                    });
                });
                // Replace the registered factory to ensure next time this action is needed, the same action is returned.
                registryHandle.destroy();
                registryHandle = actionFactories.get(_this).register(id, function () { return promise; });
                return promise;
            });
            return {
                destroy: function () {
                    this.destroy = noop;
                    destroyed = true;
                    idHandle.destroy();
                    registryHandle.destroy();
                    if (instanceHandle) {
                        instanceHandle.destroy();
                    }
                }
            };
        },
        registerCustomElementFactory: function (name, factory) {
            if (!realizeCustomElements_1.isValidName(name)) {
                throw new SyntaxError("'" + name + "' is not a valid custom element name'");
            }
            // Wrap the factory since the registry cannot store frozen factories, and dojo-compose creates
            // frozen factories…
            var wrapped = function (options) { return factory(options); };
            // Note that each custom element requires a new widget, so there's no need to replace the
            // registered factory.
            var registryHandle = customElementFactories.get(this).register(realizeCustomElements_1.normalizeName(name), wrapped);
            return {
                destroy: function () {
                    this.destroy = noop;
                    registryHandle.destroy();
                }
            };
        },
        registerStore: function (id, store) {
            var idHandle = addIdentifier(this, id);
            var instanceHandle = instanceRegistries.get(this).addStore(store, id);
            var registryHandle = storeFactories.get(this).register(id, function () { return store; });
            return {
                destroy: function () {
                    this.destroy = noop;
                    idHandle.destroy();
                    instanceHandle.destroy();
                    registryHandle.destroy();
                }
            };
        },
        registerStoreFactory: function (id, factory) {
            var _this = this;
            var idHandle = addIdentifier(this, id);
            var destroyed = false;
            var instanceHandle;
            var registryHandle = storeFactories.get(this).register(id, function () {
                var promise = Promise_1.default.resolve().then(function () {
                    // Always call the factory in a future turn. This harmonizes behavior regardless of whether the
                    // factory is registered through this method or loaded from a definition.
                    return factory();
                }).then(function (store) {
                    if (!destroyed) {
                        instanceHandle = instanceRegistries.get(_this).addStore(store, id);
                    }
                    return store;
                });
                // Replace the registered factory to ensure next time this store is needed, the same store is returned.
                registryHandle.destroy();
                registryHandle = storeFactories.get(_this).register(id, function () { return promise; });
                return promise;
            });
            return {
                destroy: function () {
                    this.destroy = noop;
                    destroyed = true;
                    idHandle.destroy();
                    registryHandle.destroy();
                    if (instanceHandle) {
                        instanceHandle.destroy();
                    }
                }
            };
        },
        registerWidget: function (id, widget) {
            var idHandle = addIdentifier(this, id);
            var instanceHandle = instanceRegistries.get(this).addWidget(widget, id);
            var registryHandle = widgetFactories.get(this).register(id, function () { return widget; });
            return {
                destroy: function () {
                    this.destroy = noop;
                    idHandle.destroy();
                    instanceHandle.destroy();
                    registryHandle.destroy();
                }
            };
        },
        registerWidgetFactory: function (id, factory) {
            var _this = this;
            var idHandle = addIdentifier(this, id);
            var destroyed = false;
            var instanceHandle;
            var registryHandle = widgetFactories.get(this).register(id, function () {
                var promise = Promise_1.default.resolve().then(function () {
                    // Always call the factory in a future turn. This harmonizes behavior regardless of whether the
                    // factory is registered through this method or loaded from a definition.
                    var _a = _this, registryProvider = _a.registryProvider, defaultWidgetStore = _a.defaultWidgetStore;
                    var options = { id: id, registryProvider: registryProvider };
                    if (defaultWidgetStore) {
                        options.stateFrom = defaultWidgetStore;
                    }
                    return factory(options);
                }).then(function (widget) {
                    if (!destroyed) {
                        instanceHandle = instanceRegistries.get(_this).addWidget(widget, id);
                    }
                    return widget;
                });
                // Replace the registered factory to ensure next time this widget is needed, the same widget is returned.
                registryHandle.destroy();
                registryHandle = widgetFactories.get(_this).register(id, function () { return promise; });
                return promise;
            });
            return {
                destroy: function () {
                    this.destroy = noop;
                    destroyed = true;
                    idHandle.destroy();
                    registryHandle.destroy();
                    if (instanceHandle) {
                        instanceHandle.destroy();
                    }
                }
            };
        },
        loadDefinition: function (_a) {
            var actions = _a.actions, customElements = _a.customElements, stores = _a.stores, widgets = _a.widgets;
            var handles = [];
            if (actions) {
                for (var _i = 0, actions_1 = actions; _i < actions_1.length; _i++) {
                    var definition = actions_1[_i];
                    var factory = factories_1.makeActionFactory(definition, midResolvers.get(this));
                    var handle = this.registerActionFactory(definition.id, factory);
                    handles.push(handle);
                }
            }
            if (customElements) {
                for (var _b = 0, customElements_1 = customElements; _b < customElements_1.length; _b++) {
                    var definition = customElements_1[_b];
                    var factory = factories_1.makeCustomElementFactory(definition, midResolvers.get(this));
                    var handle = this.registerCustomElementFactory(definition.name, factory);
                    handles.push(handle);
                }
            }
            if (stores) {
                for (var _c = 0, stores_1 = stores; _c < stores_1.length; _c++) {
                    var definition = stores_1[_c];
                    var factory = factories_1.makeStoreFactory(definition, midResolvers.get(this));
                    var handle = this.registerStoreFactory(definition.id, factory);
                    handles.push(handle);
                }
            }
            if (widgets) {
                for (var _d = 0, widgets_1 = widgets; _d < widgets_1.length; _d++) {
                    var definition = widgets_1[_d];
                    var factory = factories_1.makeWidgetFactory(definition, midResolvers.get(this), this);
                    var handle = this.registerWidgetFactory(definition.id, factory);
                    handles.push(handle);
                }
            }
            return {
                destroy: function () {
                    for (var _i = 0, _a = handles.splice(0, handles.length); _i < _a.length; _i++) {
                        var handle = _a[_i];
                        handle.destroy();
                    }
                }
            };
        },
        realize: function (root) {
            var _this = this;
            return realizeCustomElements_1.default(this.defaultWidgetStore, function (id) { return addIdentifier(_this, id); }, function (instance, id) { return registerInstance(_this, instance, id); }, publicRegistries.get(this), this.registryProvider, root);
        }
    })
        .mixin({
        mixin: {
            getAction: function (id) {
                var _this = this;
                return new Promise_1.default(function (resolve) {
                    resolve(actionFactories.get(_this).get(id)());
                });
            },
            hasAction: function (id) {
                return actionFactories.get(this).hasId(id);
            },
            identifyAction: function (action) {
                return instanceRegistries.get(this).identifyAction(action);
            },
            getCustomElementFactory: function (name) {
                return customElementFactories.get(this).get(name);
            },
            hasCustomElementFactory: function (name) {
                return customElementFactories.get(this).hasId(name);
            },
            getStore: function (id) {
                var _this = this;
                return new Promise_1.default(function (resolve) {
                    resolve(storeFactories.get(_this).get(id)());
                });
            },
            hasStore: function (id) {
                return storeFactories.get(this).hasId(id);
            },
            identifyStore: function (store) {
                return instanceRegistries.get(this).identifyStore(store);
            },
            createWidget: function (factory, options) {
                if (options === void 0) { options = {}; }
                var app = this;
                var _a = this, defaultWidgetStore = _a.defaultWidgetStore, registryProvider = _a.registryProvider;
                var _b = options.id, id = _b === void 0 ? generateWidgetId() : _b;
                // Like for custom elements, don't add the generated ID to the options.
                function configureWidget() {
                    // Ensure no other widget with this ID exists.
                    if (publicRegistries.get(app).hasWidget(id)) {
                        return Promise_1.default.reject(new Error("A widget with ID '" + id + "' already exists"));
                    }
                    if (!options.registryProvider) {
                        options.registryProvider = registryProvider;
                    }
                    if (options.id && (options.stateFrom || defaultWidgetStore)) {
                        var store = options.stateFrom = options.stateFrom || defaultWidgetStore;
                        // We will attempt to create an initial state, if it isn't present in the store
                        var state = { id: id };
                        if (options.state) {
                            lang_1.assign(state, options.state);
                        }
                        // TODO: What happens if the store rejects?
                        return store.add(state);
                    }
                }
                return Promise_1.default
                    .resolve(configureWidget())
                    .then(function () {
                    var widget = factory(options);
                    // Add the instance to the various registries the app may maintain.
                    //
                    // No need to trap registerInstance for duplicates, because we are creating new
                    // in this function
                    widget.own(registerInstance(app, widget, id));
                    return [id, widget];
                });
            },
            getWidget: function (id) {
                var _this = this;
                // Widgets either need to be resolved from a factory, or have been created when realizing
                // custom elements.
                var factories = widgetFactories.get(this);
                var instances = widgetInstances.get(this);
                return new Promise_1.default(function (resolve) {
                    // First see if a factory exists for the widget.
                    var factory;
                    var result;
                    try {
                        factory = factories.get(id);
                    }
                    catch (missingFactory) {
                        try {
                            // Otherwise try and get an existing instance.
                            result = instances.get(id);
                        }
                        catch (_) {
                            // Try creating a custom widget, but only if there is a default widget store. The default
                            // widget store should contain details on how to create the widget.
                            if (_this.defaultWidgetStore) {
                                result = createCustomWidget(_this, id)
                                    .catch(function () { throw missingFactory; });
                            }
                            else {
                                // Reject with the original error since it's probably the least confusing one.
                                result = Promise_1.default.reject(missingFactory);
                            }
                        }
                    }
                    // Resolve with the result, or call the factory and resolve with its result. If the factory throws
                    // that's fine, the returned promise will reject with an appropriate error.
                    resolve(result || factory());
                });
            },
            hasWidget: function (id) {
                // See if there is a widget factory, or else an existing custom element instance.
                return widgetFactories.get(this).hasId(id) || widgetInstances.get(this).hasId(id);
            },
            identifyWidget: function (widget) {
                return instanceRegistries.get(this).identifyWidget(widget);
            }
        },
        initialize: function (instance, _a) {
            var _b = _a === void 0 ? {} : _a, _c = _b.defaultActionStore, defaultActionStore = _c === void 0 ? null : _c, _d = _b.defaultWidgetStore, defaultWidgetStore = _d === void 0 ? null : _d, _e = _b.toAbsMid, toAbsMid = _e === void 0 ? function (moduleId) { return moduleId; } : _e;
            var publicRegistry = {
                getAction: instance.getAction.bind(instance),
                hasAction: instance.hasAction.bind(instance),
                identifyAction: instance.identifyAction.bind(instance),
                getCustomElementFactory: instance.getCustomElementFactory.bind(instance),
                hasCustomElementFactory: instance.hasCustomElementFactory.bind(instance),
                getStore: instance.getStore.bind(instance),
                hasStore: instance.hasStore.bind(instance),
                identifyStore: instance.identifyStore.bind(instance),
                createWidget: instance.createWidget.bind(instance),
                getWidget: instance.getWidget.bind(instance),
                hasWidget: instance.hasWidget.bind(instance),
                identifyWidget: instance.identifyWidget.bind(instance)
            };
            Object.freeze(publicRegistry);
            actionFactories.set(instance, new IdentityRegistry_1.default());
            customElementFactories.set(instance, new IdentityRegistry_1.default());
            identifiers.set(instance, new Set_1.default());
            storeFactories.set(instance, new IdentityRegistry_1.default());
            widgetFactories.set(instance, new IdentityRegistry_1.default());
            instanceRegistries.set(instance, new InstanceRegistry_1.default());
            midResolvers.set(instance, moduleResolver_1.default(toAbsMid));
            publicRegistries.set(instance, publicRegistry);
            registryProviders.set(instance, new RegistryProvider_1.default(publicRegistry));
            widgetInstances.set(instance, new IdentityRegistry_1.default());
            if (defaultActionStore) {
                instance.defaultActionStore = defaultActionStore;
            }
            if (defaultWidgetStore) {
                instance.defaultWidgetStore = defaultWidgetStore;
            }
        }
    });
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createApp;
});
//# sourceMappingURL=createApp.js.map