(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", 'dojo-shim/WeakMap'], factory);
    }
})(function (require, exports) {
    "use strict";
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var Type;
    (function (Type) {
        Type[Type["Action"] = 0] = "Action";
        Type[Type["Store"] = 1] = "Store";
        Type[Type["Widget"] = 2] = "Widget";
    })(Type || (Type = {}));
    ;
    var errorStrings = (_a = {},
        _a[Type.Action] = 'action',
        _a[Type.Store] = 'store',
        _a[Type.Widget] = 'widget',
        _a
    );
    var InstanceRegistry = (function () {
        function InstanceRegistry() {
            this.map = new WeakMap_1.default();
        }
        InstanceRegistry.prototype.addAction = function (action, id) {
            return this.add(action, id, Type.Action);
        };
        InstanceRegistry.prototype.identifyAction = function (action) {
            return this.identify(action, Type.Action);
        };
        InstanceRegistry.prototype.addStore = function (store, id) {
            return this.add(store, id, Type.Store);
        };
        InstanceRegistry.prototype.identifyStore = function (store) {
            return this.identify(store, Type.Store);
        };
        InstanceRegistry.prototype.addWidget = function (widget, id) {
            return this.add(widget, id, Type.Widget);
        };
        InstanceRegistry.prototype.identifyWidget = function (widget) {
            return this.identify(widget, Type.Widget);
        };
        InstanceRegistry.prototype.add = function (instance, id, type) {
            var _this = this;
            if (this.map.has(instance)) {
                var existing = this.map.get(instance);
                throw new Error("Could not add " + errorStrings[type] + ", already registered as " + errorStrings[existing.type] + " with identity " + existing.id);
            }
            this.map.set(instance, { id: id, type: type });
            var handle = {
                destroy: function () {
                    _this.map.delete(instance);
                }
            };
            return handle;
        };
        InstanceRegistry.prototype.identify = function (instance, expectedType) {
            if (!this.map.has(instance)) {
                throw new Error("Could not identify " + errorStrings[expectedType]);
            }
            var _a = this.map.get(instance), id = _a.id, type = _a.type;
            if (type !== expectedType) {
                throw new Error("Could not identify " + errorStrings[expectedType]);
            }
            return id;
        };
        return InstanceRegistry;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = InstanceRegistry;
    var _a;
});
//# sourceMappingURL=InstanceRegistry.js.map