(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    /**
     * Provides access to read-only registries for actions, stores and widgets.
     */
    var RegistryProvider = (function () {
        function RegistryProvider(combinedRegistry) {
            this.combinedRegistry = combinedRegistry;
        }
        RegistryProvider.prototype.get = function (type) {
            switch (type) {
                case 'actions':
                    return this.actionRegistry || (this.actionRegistry = {
                        get: this.combinedRegistry.getAction,
                        identify: this.combinedRegistry.identifyAction
                    });
                case 'stores':
                    return this.storeRegistry || (this.storeRegistry = {
                        get: this.combinedRegistry.getStore,
                        identify: this.combinedRegistry.identifyStore
                    });
                case 'widgets':
                    return this.widgetRegistry || (this.widgetRegistry = {
                        create: this.combinedRegistry.createWidget,
                        get: this.combinedRegistry.getWidget,
                        identify: this.combinedRegistry.identifyWidget
                    });
                default:
                    throw new Error("No such store: " + type);
            }
        };
        return RegistryProvider;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = RegistryProvider;
});
//# sourceMappingURL=RegistryProvider.js.map